using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;

using Sgml;

namespace Html2Rtf
{
	public class EntryPoint
	{
		public static void Main(string[] args)
		{
			if(args.Length == 2)
				Run(args[0], args[1]);
			else
				PrintUsage();
			
			Console.WriteLine("Press Enter for exit ...");
			Console.ReadLine();
		}

		private static void PrintUsage()
		{
			Console.WriteLine("Html2Rtf.exe htmlFile xsltFile rtfFile");
		}

		private static void Run(string htmlFile, string rtfFile)
		{
			try
			{
				// Convert html to xhtml
				XmlReader xhtml = HtmlToXhtml(htmlFile);

				// Create xml navigator for xhtml
				XPathDocument input = new XPathDocument(xhtml);

				// Create and load xsl file
				XslTransform xslt = new XslTransform();
				XmlReader xhtml2rtf = new XmlTextReader(typeof(EntryPoint).Assembly.GetManifestResourceStream("Html2Rtf.xhtml2rtf.xsl"));
				xslt.Load(xhtml2rtf, null, typeof(EntryPoint).Assembly.Evidence);

				// create xsl arguments
				XsltArgumentList args = new XsltArgumentList();
				// add helper functions for xsl transformer
				args.AddExtensionObject("http://www.lutecia.info/download/xmlns/xhtml2rtf", new Xhtml2RtfHelper());

				// transform ans write result to the file
				using(StreamWriter wr = new StreamWriter(rtfFile, false, Encoding.Default))
					xslt.Transform(input, args, wr, null);

				Console.WriteLine("Success ...");
			}
			catch(Exception ex)
			{
				Console.WriteLine("Error:");
				Console.WriteLine(ex.Message);

				if(ex.InnerException != null)
					Console.WriteLine(ex.InnerException.Message);
			}
		}

		public static XmlReader HtmlToXhtml(string path)
		{
			// create sgml reader
			SgmlReader reader = new SgmlReader();
			try
			{
				// init sgml reader
				reader.Href = path;
				reader.DocType = "HTML";
				reader.CaseFolding = CaseFolding.ToLower;

				// create stream for saving xhtml content
				MemoryStream stream = new MemoryStream(2 * 1024);

				// create xml writer for xhtml
				XmlTextWriter wr = new XmlTextWriter(stream, Encoding.UTF8);
				wr.WriteStartDocument();
			
				// convert html to xhtml
				reader.Read();
				while(!reader.EOF)
				{
					wr.WriteNode(reader, true);
				}

				// flush all data
				wr.Flush();
				
				// write xhtml file
				WriteXHmtl(Path.GetFileNameWithoutExtension(path) + ".xhtml", stream);

				// seek stream and create xml reader for xhtml
				stream.Seek(0, SeekOrigin.Begin);
				return new XmlTextReader(stream);
			}
			finally
			{
				// close sgml reader
				reader.Close();
			}
		}

		private static void WriteXHmtl(string path, MemoryStream stream)
		{
			// seek stream to begin
			stream.Seek(0, SeekOrigin.Begin);
			// and write all data to the file
			StreamReader rd = new StreamReader(stream);
			using(StreamWriter wr = new StreamWriter(path))
			{
				wr.Write(rd.ReadToEnd());
			}
		}
	}
}