using System;
using System.Collections;
using System.Text;
using System.Xml.XPath;

namespace Html2Rtf
{
	/// <summary>
	/// Xhtml2RtfHelper implements helper functions for xsl transformer.
	/// </summary>
	internal class Xhtml2RtfHelper 
	{
		int multiplyWidth = 192;
		int marginWidth = 196;
		IntCollection arrMaxLen;
		IntCollection arrTotLen;

		public string CharCode(string text)
		{
			string charCodes = "###";
			string separator = String.Empty;
			for(int i = 0; i < text.Length; i++)
			{
				charCodes += ((int)text[i]).ToString() + separator;
				separator = ",";
			}
			charCodes += "###";
			return charCodes;
		}

		public string RTFEncode(XPathNodeIterator xmlNodes, string nodeValue, int myNormalizeSpaces)
		{
			bool blnAppendParagraphBreak = false;

			// Build an array of characters
			char[] chars = nodeValue.ToCharArray();
			
			StringBuilder rtfEncoded = new StringBuilder(chars.Length);
			
			for(int i = 0; i < chars.Length; i++)
			{
				char c = chars[i];
				switch(c)
				{
					case '\\':
					case '{':
					case '}':
						// Encode backslashes, left curly bracket, right curly bracket (prefix with a backslash)
						rtfEncoded.Append('\\');
						rtfEncoded.Append(c);
						break;

					case '\u00a0':
						// Encode non-breacking space (backslash+tilda)
						rtfEncoded.Append('\\');
						rtfEncoded.Append('~');
						break;

					case '\n':
						if(myNormalizeSpaces == 2)
						{
							// Preformatted mode - use \line for all EOL characters
							rtfEncoded.Append("\\line ");

							// Check if next node is a paragraph - if yes, we will use a paragraph break INSTEAD of line break
							if(xmlNodes != null && xmlNodes.Count != 0)
							{
								XPathNavigator navigator = xmlNodes.Current;
								XPathNodeIterator firstNode = navigator.Select("following-sibling::node()[position() = 1]");
								if(firstNode != null && firstNode.MoveNext())
								{
									if(Comparer.DefaultInvariant.Compare(firstNode.Current.Name, "p") == 0)
										blnAppendParagraphBreak = true;
								}
							}
						}
						break;

					default:
						int intCharCode = c;
						if(intCharCode > 255)
						{
							// Non-ascii: encode as UNICODE (\u)
							rtfEncoded.Append("\\u");
							rtfEncoded.Append(intCharCode.ToString());
							rtfEncoded.Append("  ");
						}
						else
						{
							// TODO Handle control characters (ASCII code lesser than 32 - TAB, EOL, etc...)
							// No encoding
							rtfEncoded.Append(c);
						}
						break;
				}
			}

			if(blnAppendParagraphBreak)
			{
				// Append a paragraph break - next node is a p tag, but we are not inside a p tag (bad!)
				rtfEncoded.Append("\\par ");
			}

			return rtfEncoded.ToString();
		}
		
		public void TableCellWidthFill(XPathNodeIterator xmlNodes, int tableWidth)
		{
			if(xmlNodes != null && xmlNodes.Count != 0)
			{
				XPathNodeIterator objRowNodes = xmlNodes.Current.Select("./*");
				
				arrMaxLen = new IntCollection(objRowNodes.Count);
				int maxWordLen;
				
				while(objRowNodes.MoveNext())
				{
					XPathNavigator objRowNode = objRowNodes.Current;
					XPathNodeIterator objColNodes = objRowNode.Select("./*");

					int j = 0;
					while(objColNodes.MoveNext())
					{
						string strValue = Convert.ToString(objColNodes.Current.Value);
						//split( /[\s +]/);
						string[] arrWords = strValue.Split(' ','\f','\n','\r','\t','\v');
						
						maxWordLen = 0;

						arrTotLen = new IntCollection(objColNodes.Count);
						arrTotLen[j] = strValue.Length;
						
						for(int iWord = 0; iWord < arrWords.Length; iWord++)
						{
							if(arrWords[iWord].Length > maxWordLen)
							{
								maxWordLen = arrWords[iWord].Length;
							}
						}

						arrMaxLen[j] = arrMaxLen[j] > maxWordLen ? arrMaxLen[j] : (maxWordLen + 1);

						j++;
					}
				}
				
				if(tableWidth > 0)
				{
					int totalWidthTot = 0;
					int totalWidthMax = 0;

					for(int i = 0; i < arrMaxLen.Count; i++)
					{
						totalWidthTot += (arrTotLen[i] * multiplyWidth + 2 * marginWidth);
						totalWidthMax += (arrMaxLen[i] * multiplyWidth + 2 * marginWidth);
					}

					for(int i = 0; i < arrMaxLen.Count; i++)
					{
						arrMaxLen[i] = (arrMaxLen[i] * multiplyWidth + 2 * marginWidth) * tableWidth / totalWidthTot / multiplyWidth;
					}
				}
			}
		}
		public int GetTableColumnWidth(int iColumn, int bSum, int font_size)
		{
			multiplyWidth = font_size * 8;
			int sum = 0;
			iColumn = iColumn - 1;
			if(iColumn < arrMaxLen.Count && iColumn >= 0)
			{
				if(bSum == 0)
				{
					return (2 * marginWidth + arrMaxLen[iColumn] * multiplyWidth);
				}
				else if(bSum == 1)
				{
					return (2 * marginWidth + arrTotLen[iColumn] * multiplyWidth);
				}
				else if(bSum == 2)
				{
					for(int i = 0; i <= iColumn; i++)
					{
						sum += (2 * marginWidth + arrMaxLen[i] * multiplyWidth);
					}
					return (sum);
				}
				else if(bSum == 3)
				{
					for(int i = 0; i <= iColumn; i++)
					{
						sum += (2 * marginWidth + arrTotLen[i] * multiplyWidth);
					}
					return (sum);
				}
			}
			return 0;
		}
	}
}